@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2017  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2017-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -21,12 +21,19 @@
  */
 
 struct network_args {
-	char *name;
-	char *type;
+	const char *name;
+	const char *type;
 };
 
-bool network_is_connected(const char *path);
-void network_connect(const char *path);
+const struct proxy_interface *network_get_proxy(const char *path);
+bool network_is_connected(const struct proxy_interface *network_proxy);
+const char *network_get_type(const struct proxy_interface *network_proxy);
+const char *network_get_name(const struct proxy_interface *network_proxy);
+void network_connect(const struct proxy_interface *network_proxy);
 
-struct network_args *network_parse_args(const char *args);
-void network_args_destroy(struct network_args *network_args);
+char *network_name_completion(const struct proxy_interface *device,
+						const char *text, int state);
+
+struct l_queue *network_match_by_device_and_args(
+					const struct proxy_interface *device,
+					const struct network_args *args);
