@@ -27,10 +27,10 @@
 #include <stdio.h>
 #include <wchar.h>
 
-#include "util.h"
 #include "strv.h"
 #include "utf8.h"
 #include "private.h"
+#include "useful.h"
 
 /**
  * SECTION:utf8
@@ -93,12 +93,12 @@ LIB_EXPORT int l_utf8_get_codepoint(const char *str, size_t len, wchar_t *cp)
 	if (len == 0)
 		return 0;
 
-	if (str[0] > 0) {
+	if ((signed char) str[0] > 0) {
 		*cp = str[0];
 		return 1;
 	}
 
-	expect_bytes = __builtin_clz(~(str[0] << 24));
+	expect_bytes = __builtin_clz(~((unsigned char)str[0] << 24));
 
 	if (expect_bytes < 2 || expect_bytes > 4)
 		goto error;
@@ -109,7 +109,7 @@ LIB_EXPORT int l_utf8_get_codepoint(const char *str, size_t len, wchar_t *cp)
 	val = str[0] & (0xff >> (expect_bytes + 1));
 
 	for (i = 1; i < expect_bytes; i++) {
-		if ((str[i] & 0xc0) == 0)
+		if ((str[i] & 0xc0) != 0x80)
 			goto error;
 
 		val <<= 6;
