@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2014  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -176,7 +176,8 @@ static size_t rta_add(void *rta_buf, unsigned short type, uint16_t len,
 
 	rta->rta_len = rta_len;
 	rta->rta_type = type;
-	memcpy(RTA_DATA(rta), data, len);
+	if (len)
+		memcpy(RTA_DATA(rta), data, len);
 
 	return RTA_SPACE(len);
 }
@@ -221,6 +222,12 @@ static struct l_netlink *rtm_interface_send_message(struct l_netlink *rtnl,
 			return NULL;
 	}
 
+	if (!rtnl)
+		rtnl = l_netlink_new(NETLINK_ROUTE);
+
+	if (!rtnl)
+		return NULL;
+
 	bufsize = NLMSG_LENGTH(sizeof(struct ifinfomsg)) +
 		RTA_SPACE(ifname_len) + RTA_SPACE(0) +
 		RTA_SPACE(nlmon_type_len);
@@ -231,12 +238,6 @@ static struct l_netlink *rtm_interface_send_message(struct l_netlink *rtnl,
 	rtmmsg->ifi_family = AF_UNSPEC;
 	rtmmsg->ifi_change = ~0;
 
-	if (!rtnl)
-		rtnl = l_netlink_new(NETLINK_ROUTE);
-
-	if (!rtnl)
-		return NULL;
-
 	rta_buf = rtmmsg + 1;
 
 	if (ifname)
@@ -250,7 +251,6 @@ static struct l_netlink *rtm_interface_send_message(struct l_netlink *rtnl,
 	linkinfo_rta->rta_len = rta_buf - (void *) linkinfo_rta;
 
 	switch (rtm_msg_type) {
-
 	case RTM_NEWLINK:
 		rtmmsg->ifi_flags = IFF_UP | IFF_ALLMULTI | IFF_NOARP;
 
@@ -275,8 +275,8 @@ static struct l_netlink *rtm_interface_send_message(struct l_netlink *rtnl,
 
 	default:
 		l_netlink_destroy(rtnl);
-		l_free(rtmmsg);
-		return NULL;
+		rtnl = NULL;
+		break;
 	}
 
 	l_free(rtmmsg);
@@ -671,6 +671,7 @@ static void usage(void)
 		"\t-n, --nortnl           Don't show RTNL output\n"
 		"\t-y, --nowiphy          Don't show 'New Wiphy' output\n"
 		"\t-s, --noscan           Don't show scan result output\n"
+		"\t-e, --noies            Don't show IEs except SSID\n"
 		"\t-h, --help             Show help options\n");
 }
 
@@ -683,6 +684,7 @@ static const struct option main_options[] = {
 	{ "nortnl",    no_argument,       NULL, 'n' },
 	{ "nowiphy",   no_argument,       NULL, 'y' },
 	{ "noscan",    no_argument,       NULL, 's' },
+	{ "noies",     no_argument,       NULL, 'e' },
 	{ "version",   no_argument,       NULL, 'v' },
 	{ "help",      no_argument,       NULL, 'h' },
 	{ }
@@ -746,6 +748,9 @@ int main(int argc, char *argv[])
 		case 's':
 			config.noscan = true;
 			break;
+		case 'e':
+			config.noies = true;
+			break;
 		case 'v':
 			printf("%s\n", VERSION);
 			return EXIT_SUCCESS;
