@@ -37,6 +37,7 @@
 
 #define SAE_RETRANSMIT_TIMEOUT	2
 #define SAE_SYNC_MAX		3
+#define SAE_MAX_ASSOC_RETRY	3
 
 enum sae_state {
 	SAE_STATE_NOTHING = 0,
@@ -73,6 +74,7 @@ struct sae_sm {
 	uint16_t rc;
 	/* remote peer */
 	uint8_t peer[6];
+	uint8_t assoc_retry;
 
 	sae_tx_authenticate_func_t tx_auth;
 	sae_tx_associate_func_t tx_assoc;
@@ -550,7 +552,7 @@ static int sae_process_commit(struct sae_sm *sm, const uint8_t *from,
 	/*
 	 * IEEE 802.11-2016 - Section 12.4.4.2.1 ECC group definition
 	 * ECC groups make use of a mapping function, F, that maps a
-	 * point (x, y) that satisfies the curve equation to its x-coordinate—
+	 * point (x, y) that satisfies the curve equation to its x-coordinate-
 	 * i.e., if P = (x, y) then F(P) = x.
 	 */
 	klen = l_ecc_point_get_x(k_point, k, sizeof(k));
@@ -670,31 +672,16 @@ static bool sae_send_commit(struct sae_sm *sm, bool retry)
 	return true;
 }
 
-static bool sae_timeout(struct auth_proto *ap)
+static bool sae_assoc_timeout(struct auth_proto *ap)
 {
 	struct sae_sm *sm = l_container_of(ap, struct sae_sm, ap);
 
-	/* regardless of state, reject if sync exceeds max */
-	if (sm->sync > SAE_SYNC_MAX) {
-		sae_reject_authentication(sm, MMPDU_REASON_CODE_UNSPECIFIED);
+	if (sm->assoc_retry >= SAE_MAX_ASSOC_RETRY)
 		return false;
-	}
 
-	sm->sync++;
+	sm->assoc_retry++;
 
-	switch (sm->state) {
-	case SAE_STATE_COMMITTED:
-		sae_send_commit(sm, true);
-		break;
-	case SAE_STATE_CONFIRMED:
-		sm->sc++;
-		sae_send_confirm(sm);
-		break;
-	default:
-		/* should never happen */
-		l_error("SAE timeout in bad state %u", sm->state);
-		return false;
-	}
+	sm->tx_assoc(sm->user_data);
 
 	return true;
 }
@@ -1178,7 +1165,7 @@ struct auth_proto *sae_sm_new(struct handshake_state *hs,
 	sm->ap.free = sae_free;
 	sm->ap.rx_authenticate = sae_rx_authenticate;
 	sm->ap.rx_associate = sae_rx_associate;
-	sm->ap.auth_timeout = sae_timeout;
+	sm->ap.assoc_timeout = sae_assoc_timeout;
 
 	return &sm->ap;
 }
